# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ListSecretsRequestOrderBy
from .types import Product
from .types import SecretStatus
from .types import SecretVersionStatus
from .types import AccessSecretVersionResponse
from .types import ListSecretVersionsResponse
from .types import ListSecretsResponse
from .types import PasswordGenerationParams
from .types import Secret
from .types import SecretVersion
from .api import SecretV1Alpha1API

__all__ = [
    "ListSecretsRequestOrderBy",
    "Product",
    "SecretStatus",
    "SecretVersionStatus",
    "AccessSecretVersionResponse",
    "ListSecretVersionsResponse",
    "ListSecretsResponse",
    "PasswordGenerationParams",
    "Secret",
    "SecretVersion",
    "SecretV1Alpha1API",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from dateutil import parser
from .types import (
    CreateEmailRequestAddress,
    CreateEmailRequestAttachment,
    CreateEmailResponse,
    Domain,
    DomainStatistics,
    Email,
    EmailTry,
    ListDomainsResponse,
    ListEmailsResponse,
    Statistics,
    CreateEmailRequest,
    CreateDomainRequest,
)


def unmarshal_DomainStatistics(data: Any) -> DomainStatistics:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'DomainStatistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("canceled_count")
    args["canceled_count"] = field

    field = data.get("failed_count")
    args["failed_count"] = field

    field = data.get("sent_count")
    args["sent_count"] = field

    field = data.get("total_count")
    args["total_count"] = field

    return DomainStatistics(**args)


def unmarshal_EmailTry(data: Any) -> EmailTry:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'EmailTry' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("code")
    args["code"] = field

    field = data.get("message")
    args["message"] = field

    field = data.get("rank")
    args["rank"] = field

    field = data.get("tried_at")
    args["tried_at"] = parser.isoparse(field) if type(field) is str else field

    return EmailTry(**args)


def unmarshal_Domain(data: Any) -> Domain:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Domain' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("dkim_config")
    args["dkim_config"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("last_error")
    args["last_error"] = field

    field = data.get("last_valid_at")
    args["last_valid_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("name")
    args["name"] = field

    field = data.get("next_check_at")
    args["next_check_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("region")
    args["region"] = field

    field = data.get("revoked_at")
    args["revoked_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("spf_config")
    args["spf_config"] = field

    field = data.get("statistics")
    args["statistics"] = (
        unmarshal_DomainStatistics(field) if field is not None else None
    )

    field = data.get("status")
    args["status"] = field

    return Domain(**args)


def unmarshal_Email(data: Any) -> Email:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Email' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id")
    args["id"] = field

    field = data.get("last_tries")
    args["last_tries"] = [unmarshal_EmailTry(v) for v in data["last_tries"]]

    field = data.get("mail_from")
    args["mail_from"] = field

    field = data.get("mail_rcpt")
    args["mail_rcpt"] = field

    field = data.get("message_id")
    args["message_id"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("rcpt_to")
    args["rcpt_to"] = field

    field = data.get("rcpt_type")
    args["rcpt_type"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("status_details")
    args["status_details"] = field

    field = data.get("subject")
    args["subject"] = field

    field = data.get("try_count")
    args["try_count"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return Email(**args)


def unmarshal_CreateEmailResponse(data: Any) -> CreateEmailResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'CreateEmailResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("emails")
    args["emails"] = [unmarshal_Email(v) for v in data["emails"]]

    return CreateEmailResponse(**args)


def unmarshal_ListDomainsResponse(data: Any) -> ListDomainsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("domains")
    args["domains"] = [unmarshal_Domain(v) for v in data["domains"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListDomainsResponse(**args)


def unmarshal_ListEmailsResponse(data: Any) -> ListEmailsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListEmailsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("emails")
    args["emails"] = [unmarshal_Email(v) for v in data["emails"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListEmailsResponse(**args)


def unmarshal_Statistics(data: Any) -> Statistics:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Statistics' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("canceled_count")
    args["canceled_count"] = field

    field = data.get("failed_count")
    args["failed_count"] = field

    field = data.get("new_count")
    args["new_count"] = field

    field = data.get("sending_count")
    args["sending_count"] = field

    field = data.get("sent_count")
    args["sent_count"] = field

    field = data.get("total_count")
    args["total_count"] = field

    return Statistics(**args)


def marshal_CreateEmailRequestAddress(
    request: CreateEmailRequestAddress,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "email": request.email,
        "name": request.name,
    }


def marshal_CreateEmailRequestAttachment(
    request: CreateEmailRequestAttachment,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "content": request.content,
        "name": request.name,
        "type": request.type_,
    }


def marshal_CreateDomainRequest(
    request: CreateDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "accept_tos": request.accept_tos,
        "domain_name": request.domain_name,
        "project_id": request.project_id or defaults.default_project_id,
    }


def marshal_CreateEmailRequest(
    request: CreateEmailRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "attachments": [
            marshal_CreateEmailRequestAttachment(v, defaults)
            for v in request.attachments
        ]
        if request.attachments is not None
        else None,
        "bcc": [marshal_CreateEmailRequestAddress(v, defaults) for v in request.bcc]
        if request.bcc is not None
        else None,
        "cc": [marshal_CreateEmailRequestAddress(v, defaults) for v in request.cc]
        if request.cc is not None
        else None,
        "from": marshal_CreateEmailRequestAddress(request.from_, defaults)
        if request.from_ is not None
        else None,
        "html": request.html,
        "project_id": request.project_id or defaults.default_project_id,
        "send_before": request.send_before,
        "subject": request.subject,
        "text": request.text,
        "to": [marshal_CreateEmailRequestAddress(v, defaults) for v in request.to]
        if request.to is not None
        else None,
    }



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from dateutil import parser
from .types import (
    GetImageResponse,
    GetVersionResponse,
    Image,
    ListImagesResponse,
    ListVersionsResponse,
    LocalImage,
    Organization,
    Version,
)


def unmarshal_LocalImage(data: Any) -> LocalImage:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'LocalImage' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("arch")
    args["arch"] = field

    field = data.get("compatible_commercial_types")
    args["compatible_commercial_types"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("zone")
    args["zone"] = field

    return LocalImage(**args)


def unmarshal_Organization(data: Any) -> Organization:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Organization' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    return Organization(**args)


def unmarshal_Version(data: Any) -> Version:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Version' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("creation_date")
    args["creation_date"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id")
    args["id"] = field

    field = data.get("local_images")
    args["local_images"] = [unmarshal_LocalImage(v) for v in data["local_images"]]

    field = data.get("modification_date")
    args["modification_date"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("name")
    args["name"] = field

    return Version(**args)


def unmarshal_Image(data: Any) -> Image:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Image' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("categories")
    args["categories"] = field

    field = data.get("creation_date")
    args["creation_date"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("current_public_version")
    args["current_public_version"] = field

    field = data.get("description")
    args["description"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("label")
    args["label"] = field

    field = data.get("logo")
    args["logo"] = field

    field = data.get("modification_date")
    args["modification_date"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("name")
    args["name"] = field

    field = data.get("organization")
    args["organization"] = unmarshal_Organization(field) if field is not None else None

    field = data.get("valid_until")
    args["valid_until"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("versions")
    args["versions"] = [unmarshal_Version(v) for v in data["versions"]]

    return Image(**args)


def unmarshal_GetImageResponse(data: Any) -> GetImageResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'GetImageResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("image")
    args["image"] = unmarshal_Image(field) if field is not None else None

    return GetImageResponse(**args)


def unmarshal_GetVersionResponse(data: Any) -> GetVersionResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'GetVersionResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("version")
    args["version"] = unmarshal_Version(field) if field is not None else None

    return GetVersionResponse(**args)


def unmarshal_ListImagesResponse(data: Any) -> ListImagesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListImagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("images")
    args["images"] = [unmarshal_Image(v) for v in data["images"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListImagesResponse(**args)


def unmarshal_ListVersionsResponse(data: Any) -> ListVersionsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListVersionsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count")
    args["total_count"] = field

    field = data.get("versions")
    args["versions"] = [unmarshal_Version(v) for v in data["versions"]]

    return ListVersionsResponse(**args)



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional

from scaleway_core.bridge import (
    Money,
)


class ContactEmailStatus(str, Enum):
    EMAIL_STATUS_UNKNOWN = "email_status_unknown"
    VALIDATED = "validated"
    NOT_VALIDATED = "not_validated"
    INVALID_EMAIL = "invalid_email"

    def __str__(self) -> str:
        return str(self.value)


class ContactExtensionFRMode(str, Enum):
    MODE_UNKNOWN = "mode_unknown"
    INDIVIDUAL = "individual"
    COMPANY_IDENTIFICATION_CODE = "company_identification_code"
    DUNS = "duns"
    LOCAL = "local"
    ASSOCIATION = "association"
    TRADEMARK = "trademark"
    CODE_AUTH_AFNIC = "code_auth_afnic"

    def __str__(self) -> str:
        return str(self.value)


class ContactExtensionNLLegalForm(str, Enum):
    LEGAL_FORM_UNKNOWN = "legal_form_unknown"
    OTHER = "other"
    NON_DUTCH_EU_COMPANY = "non_dutch_eu_company"
    NON_DUTCH_LEGAL_FORM_ENTERPRISE_SUBSIDIARY = (
        "non_dutch_legal_form_enterprise_subsidiary"
    )
    LIMITED_COMPANY = "limited_company"
    LIMITED_COMPANY_IN_FORMATION = "limited_company_in_formation"
    COOPERATIVE = "cooperative"
    LIMITED_PARTNERSHIP = "limited_partnership"
    SOLE_COMPANY = "sole_company"
    EUROPEAN_ECONOMIC_INTEREST_GROUP = "european_economic_interest_group"
    RELIGIOUS_ENTITY = "religious_entity"
    PARTNERSHIP = "partnership"
    PUBLIC_COMPANY = "public_company"
    MUTUAL_BENEFIT_COMPANY = "mutual_benefit_company"
    RESIDENTIAL = "residential"
    SHIPPING_COMPANY = "shipping_company"
    FOUNDATION = "foundation"
    ASSOCIATION = "association"
    TRADING_PARTNERSHIP = "trading_partnership"
    NATURAL_PERSON = "natural_person"

    def __str__(self) -> str:
        return str(self.value)


class ContactLegalForm(str, Enum):
    LEGAL_FORM_UNKNOWN = "legal_form_unknown"
    INDIVIDUAL = "individual"
    CORPORATE = "corporate"
    ASSOCIATION = "association"
    OTHER = "other"

    def __str__(self) -> str:
        return str(self.value)


class DNSZoneStatus(str, Enum):
    UNKNOWN = "unknown"
    ACTIVE = "active"
    PENDING = "pending"
    ERROR = "error"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class DSRecordAlgorithm(str, Enum):
    RSAMD5 = "rsamd5"
    DH = "dh"
    DSA = "dsa"
    RSASHA1 = "rsasha1"
    DSA_NSEC3_SHA1 = "dsa_nsec3_sha1"
    RSASHA1_NSEC3_SHA1 = "rsasha1_nsec3_sha1"
    RSASHA256 = "rsasha256"
    RSASHA512 = "rsasha512"
    ECC_GOST = "ecc_gost"
    ECDSAP256SHA256 = "ecdsap256sha256"
    ECDSAP384SHA384 = "ecdsap384sha384"
    ED25519 = "ed25519"
    ED448 = "ed448"

    def __str__(self) -> str:
        return str(self.value)


class DSRecordDigestType(str, Enum):
    SHA_1 = "sha_1"
    SHA_256 = "sha_256"
    GOST_R_34_11_94 = "gost_r_34_11_94"
    SHA_384 = "sha_384"

    def __str__(self) -> str:
        return str(self.value)


class DomainFeatureStatus(str, Enum):
    FEATURE_STATUS_UNKNOWN = "feature_status_unknown"
    ENABLING = "enabling"
    ENABLED = "enabled"
    DISABLING = "disabling"
    DISABLED = "disabled"

    def __str__(self) -> str:
        return str(self.value)


class DomainRecordHTTPServiceConfigStrategy(str, Enum):
    RANDOM = "random"
    HASHED = "hashed"
    ALL = "all"

    def __str__(self) -> str:
        return str(self.value)


class DomainRecordType(str, Enum):
    UNKNOWN = "unknown"
    A = "A"
    AAAA = "AAAA"
    CNAME = "CNAME"
    TXT = "TXT"
    SRV = "SRV"
    TLSA = "TLSA"
    MX = "MX"
    NS = "NS"
    PTR = "PTR"
    CAA = "CAA"
    ALIAS = "ALIAS"
    LOC = "LOC"
    SSHFP = "SSHFP"
    HINFO = "HINFO"
    RP = "RP"
    URI = "URI"
    DS = "DS"
    NAPTR = "NAPTR"
    DNAME = "DNAME"

    def __str__(self) -> str:
        return str(self.value)


class DomainRegistrationStatusTransferStatus(str, Enum):
    STATUS_UNKNOWN = "status_unknown"
    PENDING = "pending"
    WAITING_VOTE = "waiting_vote"
    REJECTED = "rejected"
    PROCESSING = "processing"
    DONE = "done"

    def __str__(self) -> str:
        return str(self.value)


class DomainStatus(str, Enum):
    STATUS_UNKNOWN = "status_unknown"
    ACTIVE = "active"
    CREATING = "creating"
    CREATE_ERROR = "create_error"
    RENEWING = "renewing"
    RENEW_ERROR = "renew_error"
    XFERING = "xfering"
    XFER_ERROR = "xfer_error"
    EXPIRED = "expired"
    EXPIRING = "expiring"
    UPDATING = "updating"
    CHECKING = "checking"
    LOCKED = "locked"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class HostStatus(str, Enum):
    UNKNOWN_STATUS = "unknown_status"
    ACTIVE = "active"
    UPDATING = "updating"
    DELETING = "deleting"

    def __str__(self) -> str:
        return str(self.value)


class LanguageCode(str, Enum):
    UNKNOWN_LANGUAGE_CODE = "unknown_language_code"
    EN_US = "en_US"
    FR_FR = "fr_FR"
    DE_DE = "de_DE"

    def __str__(self) -> str:
        return str(self.value)


class ListDNSZoneRecordsRequestOrderBy(str, Enum):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListDNSZonesRequestOrderBy(str, Enum):
    DOMAIN_ASC = "domain_asc"
    DOMAIN_DESC = "domain_desc"
    SUBDOMAIN_ASC = "subdomain_asc"
    SUBDOMAIN_DESC = "subdomain_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListDomainsRequestOrderBy(str, Enum):
    DOMAIN_ASC = "domain_asc"
    DOMAIN_DESC = "domain_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListRenewableDomainsRequestOrderBy(str, Enum):
    DOMAIN_ASC = "domain_asc"
    DOMAIN_DESC = "domain_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListTasksRequestOrderBy(str, Enum):
    DOMAIN_DESC = "domain_desc"
    DOMAIN_ASC = "domain_asc"
    TYPE_ASC = "type_asc"
    TYPE_DESC = "type_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class RawFormat(str, Enum):
    UNKNOWN_RAW_FORMAT = "unknown_raw_format"
    BIND = "bind"

    def __str__(self) -> str:
        return str(self.value)


class RenewableDomainStatus(str, Enum):
    UNKNOWN = "unknown"
    RENEWABLE = "renewable"
    LATE_RENEWEABLE = "late_reneweable"
    NOT_RENEWABLE = "not_renewable"

    def __str__(self) -> str:
        return str(self.value)


class SSLCertificateStatus(str, Enum):
    UNKNOWN = "unknown"
    NEW = "new"
    PENDING = "pending"
    SUCCESS = "success"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)


class TaskStatus(str, Enum):
    UNAVAILABLE = "unavailable"
    NEW = "new"
    WAITING_PAYMENT = "waiting_payment"
    PENDING = "pending"
    SUCCESS = "success"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)


class TaskType(str, Enum):
    UNKNOWN = "unknown"
    CREATE_DOMAIN = "create_domain"
    CREATE_EXTERNAL_DOMAIN = "create_external_domain"
    RENEW_DOMAIN = "renew_domain"
    TRANSFER_DOMAIN = "transfer_domain"
    TRADE_DOMAIN = "trade_domain"
    LOCK_DOMAIN_TRANSFER = "lock_domain_transfer"
    UNLOCK_DOMAIN_TRANSFER = "unlock_domain_transfer"
    ENABLE_DNSSEC = "enable_dnssec"
    DISABLE_DNSSEC = "disable_dnssec"
    UPDATE_DOMAIN = "update_domain"
    UPDATE_CONTACT = "update_contact"
    DELETE_DOMAIN = "delete_domain"
    CANCEL_TASK = "cancel_task"
    GENERATE_SSL_CERTIFICATE = "generate_ssl_certificate"
    RENEW_SSL_CERTIFICATE = "renew_ssl_certificate"
    SEND_MESSAGE = "send_message"
    DELETE_DOMAIN_EXPIRED = "delete_domain_expired"
    DELETE_EXTERNAL_DOMAIN = "delete_external_domain"
    CREATE_HOST = "create_host"
    UPDATE_HOST = "update_host"
    DELETE_HOST = "delete_host"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class AvailableDomain:
    domain: str

    available: bool

    tld: Optional[Tld]


@dataclass
class CheckContactsCompatibilityResponse:
    """
    Check contacts compatibility response.
    """

    compatible: bool

    owner_check_result: Optional[CheckContactsCompatibilityResponseContactCheckResult]

    administrative_check_result: Optional[
        CheckContactsCompatibilityResponseContactCheckResult
    ]

    technical_check_result: Optional[
        CheckContactsCompatibilityResponseContactCheckResult
    ]


@dataclass
class CheckContactsCompatibilityResponseContactCheckResult:
    compatible: bool

    error_message: Optional[str]


@dataclass
class ClearDNSZoneRecordsResponse:
    """
    Clear dns zone records response.
    """


@dataclass
class Contact:
    """
    Contact.
    """

    id: str

    legal_form: ContactLegalForm

    firstname: str

    lastname: str

    company_name: str

    email: str

    email_alt: str

    phone_number: str

    fax_number: str

    address_line_1: str

    address_line_2: str

    zip: str

    city: str

    country: str

    vat_identification_code: str

    company_identification_code: str

    lang: LanguageCode

    resale: bool

    questions: Optional[List[ContactQuestion]]
    """
    :deprecated
    """

    extension_fr: Optional[ContactExtensionFR]

    extension_eu: Optional[ContactExtensionEU]

    whois_opt_in: bool

    email_status: ContactEmailStatus

    state: str

    extension_nl: Optional[ContactExtensionNL]


@dataclass
class ContactExtensionEU:
    european_citizenship: str


@dataclass
class ContactExtensionFR:
    mode: ContactExtensionFRMode

    individual_info: Optional[ContactExtensionFRIndividualInfo]
    """
    One-of ('mode_fields'): at most one of 'individual_info', 'duns_info', 'association_info', 'trademark_info', 'code_auth_afnic_info' could be set.
    """

    duns_info: Optional[ContactExtensionFRDunsInfo]
    """
    One-of ('mode_fields'): at most one of 'individual_info', 'duns_info', 'association_info', 'trademark_info', 'code_auth_afnic_info' could be set.
    """

    association_info: Optional[ContactExtensionFRAssociationInfo]
    """
    One-of ('mode_fields'): at most one of 'individual_info', 'duns_info', 'association_info', 'trademark_info', 'code_auth_afnic_info' could be set.
    """

    trademark_info: Optional[ContactExtensionFRTrademarkInfo]
    """
    One-of ('mode_fields'): at most one of 'individual_info', 'duns_info', 'association_info', 'trademark_info', 'code_auth_afnic_info' could be set.
    """

    code_auth_afnic_info: Optional[ContactExtensionFRCodeAuthAfnicInfo]
    """
    One-of ('mode_fields'): at most one of 'individual_info', 'duns_info', 'association_info', 'trademark_info', 'code_auth_afnic_info' could be set.
    """


@dataclass
class ContactExtensionFRAssociationInfo:
    publication_jo: Optional[datetime]

    publication_jo_page: int


@dataclass
class ContactExtensionFRCodeAuthAfnicInfo:
    code_auth_afnic: str


@dataclass
class ContactExtensionFRDunsInfo:
    duns_id: str

    local_id: str


@dataclass
class ContactExtensionFRIndividualInfo:
    whois_opt_in: bool


@dataclass
class ContactExtensionFRTrademarkInfo:
    trademark_inpi: str


@dataclass
class ContactExtensionNL:
    legal_form: ContactExtensionNLLegalForm

    legal_form_registration_number: str


@dataclass
class ContactQuestion:
    question: str

    answer: str


@dataclass
class ContactRoles:
    contact: Optional[Contact]

    roles: Dict[str, ContactRolesRoles]


@dataclass
class ContactRolesRoles:
    is_owner: bool

    is_administrative: bool

    is_technical: bool


@dataclass
class DNSZone:
    domain: str

    subdomain: str

    ns: List[str]

    ns_default: List[str]

    ns_master: List[str]

    status: DNSZoneStatus

    message: Optional[str]

    updated_at: Optional[datetime]

    project_id: str


@dataclass
class DNSZoneVersion:
    id: str

    created_at: Optional[datetime]


@dataclass
class DSRecord:
    key_id: int

    algorithm: DSRecordAlgorithm

    digest: Optional[DSRecordDigest]
    """
    One-of ('type_'): at most one of 'digest', 'public_key' could be set.
    """

    public_key: Optional[DSRecordPublicKey]
    """
    One-of ('type_'): at most one of 'digest', 'public_key' could be set.
    """


@dataclass
class DSRecordDigest:
    type_: DSRecordDigestType

    digest: str

    public_key: Optional[DSRecordPublicKey]


@dataclass
class DSRecordPublicKey:
    key: str


@dataclass
class DeleteDNSZoneResponse:
    """
    Delete dns zone response.
    """


@dataclass
class DeleteExternalDomainResponse:
    """
    Delete external domain response.
    """


@dataclass
class DeleteSSLCertificateResponse:
    """
    Delete ssl certificate response.
    """


@dataclass
class Domain:
    """
    Domain.
    """

    domain: str

    organization_id: str

    project_id: str

    auto_renew_status: DomainFeatureStatus

    dnssec: Optional[DomainDNSSEC]

    epp_code: List[str]

    expired_at: Optional[datetime]

    updated_at: Optional[datetime]

    registrar: str

    is_external: bool

    status: DomainStatus

    dns_zones: List[DNSZone]

    owner_contact: Optional[Contact]

    technical_contact: Optional[Contact]

    administrative_contact: Optional[Contact]

    external_domain_registration_status: Optional[
        DomainRegistrationStatusExternalDomain
    ]
    """
    One-of ('registration_status'): at most one of 'external_domain_registration_status', 'transfer_registration_status' could be set.
    """

    transfer_registration_status: Optional[DomainRegistrationStatusTransfer]
    """
    One-of ('registration_status'): at most one of 'external_domain_registration_status', 'transfer_registration_status' could be set.
    """

    tld: Optional[Tld]


@dataclass
class DomainDNSSEC:
    status: DomainFeatureStatus

    ds_records: List[DSRecord]


@dataclass
class DomainRecord:
    data: str

    name: str

    priority: int

    ttl: int

    type_: DomainRecordType

    comment: Optional[str]

    geo_ip_config: Optional[DomainRecordGeoIPConfig]
    """
    One-of ('dynamic_data'): at most one of 'geo_ip_config', 'http_service_config', 'weighted_config', 'view_config' could be set.
    """

    http_service_config: Optional[DomainRecordHTTPServiceConfig]
    """
    One-of ('dynamic_data'): at most one of 'geo_ip_config', 'http_service_config', 'weighted_config', 'view_config' could be set.
    """

    weighted_config: Optional[DomainRecordWeightedConfig]
    """
    One-of ('dynamic_data'): at most one of 'geo_ip_config', 'http_service_config', 'weighted_config', 'view_config' could be set.
    """

    view_config: Optional[DomainRecordViewConfig]
    """
    One-of ('dynamic_data'): at most one of 'geo_ip_config', 'http_service_config', 'weighted_config', 'view_config' could be set.
    """

    id: str


@dataclass
class DomainRecordGeoIPConfig:
    matches: List[DomainRecordGeoIPConfigMatch]

    default: str


@dataclass
class DomainRecordGeoIPConfigMatch:
    countries: List[str]

    continents: List[str]

    data: str


@dataclass
class DomainRecordHTTPServiceConfig:
    ips: List[str]

    must_contain: Optional[str]

    url: str

    user_agent: Optional[str]

    strategy: DomainRecordHTTPServiceConfigStrategy


@dataclass
class DomainRecordViewConfig:
    views: List[DomainRecordViewConfigView]


@dataclass
class DomainRecordViewConfigView:
    subnet: str

    data: str


@dataclass
class DomainRecordWeightedConfig:
    weighted_ips: List[DomainRecordWeightedConfigWeightedIP]


@dataclass
class DomainRecordWeightedConfigWeightedIP:
    ip: str

    weight: int


@dataclass
class DomainRegistrationStatusExternalDomain:
    validation_token: str


@dataclass
class DomainRegistrationStatusTransfer:
    status: DomainRegistrationStatusTransferStatus

    vote_current_owner: bool

    vote_new_owner: bool


@dataclass
class DomainSummary:
    domain: str

    project_id: str

    auto_renew_status: DomainFeatureStatus

    dnssec_status: DomainFeatureStatus

    epp_code: List[str]

    expired_at: Optional[datetime]

    updated_at: Optional[datetime]

    registrar: str

    is_external: bool

    status: DomainStatus

    external_domain_registration_status: Optional[
        DomainRegistrationStatusExternalDomain
    ]
    """
    One-of ('registration_status'): at most one of 'external_domain_registration_status', 'transfer_registration_status' could be set.
    """

    transfer_registration_status: Optional[DomainRegistrationStatusTransfer]
    """
    One-of ('registration_status'): at most one of 'external_domain_registration_status', 'transfer_registration_status' could be set.
    """

    organization_id: str


@dataclass
class GetDNSZoneTsigKeyResponse:
    """
    Get dns zone tsig key response.
    """

    name: str

    key: str

    algorithm: str


@dataclass
class GetDNSZoneVersionDiffResponse:
    """
    Get dns zone version diff response.
    """

    changes: List[RecordChange]


@dataclass
class GetDomainAuthCodeResponse:
    """
    Get domain auth code response.
    """

    auth_code: str


@dataclass
class Host:
    domain: str

    name: str

    ips: List[str]

    status: HostStatus


@dataclass
class ImportProviderDNSZoneRequestOnlineV1:
    token: str


@dataclass
class ImportProviderDNSZoneResponse:
    """
    Import provider dns zone response.
    """

    records: List[DomainRecord]


@dataclass
class ImportRawDNSZoneRequestAXFRSource:
    name_server: str

    tsig_key: Optional[ImportRawDNSZoneRequestTsigKey]


@dataclass
class ImportRawDNSZoneRequestBindSource:
    content: str


@dataclass
class ImportRawDNSZoneRequestTsigKey:
    name: str

    key: str

    algorithm: str


@dataclass
class ImportRawDNSZoneResponse:
    """
    Import raw dns zone response.
    """

    records: List[DomainRecord]


@dataclass
class ListContactsResponse:
    """
    List contacts response.
    """

    total_count: int

    contacts: List[ContactRoles]


@dataclass
class ListDNSZoneNameserversResponse:
    """
    List dns zone nameservers response.
    """

    ns: List[Nameserver]
    """
    DNS zone name servers returned.
    """


@dataclass
class ListDNSZoneRecordsResponse:
    """
    List dns zone records response.
    """

    total_count: int
    """
    Total number of DNS zone records.
    """

    records: List[DomainRecord]
    """
    Paginated returned DNS zone records.
    """


@dataclass
class ListDNSZoneVersionRecordsResponse:
    """
    List dns zone version records response.
    """

    total_count: int
    """
    Total number of DNS zones versions records.
    """

    records: List[DomainRecord]


@dataclass
class ListDNSZoneVersionsResponse:
    """
    List dns zone versions response.
    """

    total_count: int
    """
    Total number of DNS zones versions.
    """

    versions: List[DNSZoneVersion]


@dataclass
class ListDNSZonesResponse:
    """
    List dns zones response.
    """

    total_count: int
    """
    Total number of DNS zones matching the requested criteria.
    """

    dns_zones: List[DNSZone]
    """
    Paginated returned DNS zones.
    """


@dataclass
class ListDomainHostsResponse:
    """
    List domain hosts response.
    """

    total_count: int

    hosts: List[Host]


@dataclass
class ListDomainsResponse:
    """
    List domains response.
    """

    total_count: int

    domains: List[DomainSummary]


@dataclass
class ListRenewableDomainsResponse:
    """
    List renewable domains response.
    """

    total_count: int

    domains: List[RenewableDomain]


@dataclass
class ListSSLCertificatesResponse:
    """
    List ssl certificates response.
    """

    total_count: int

    certificates: List[SSLCertificate]


@dataclass
class ListTasksResponse:
    """
    List tasks response.
    """

    total_count: int

    tasks: List[Task]


@dataclass
class Nameserver:
    name: str

    ip: List[str]


@dataclass
class NewContact:
    legal_form: ContactLegalForm

    firstname: str

    lastname: str

    company_name: Optional[str]

    email: str

    email_alt: Optional[str]

    phone_number: str

    fax_number: Optional[str]

    address_line_1: str

    address_line_2: Optional[str]

    zip: str

    city: str

    country: str

    vat_identification_code: Optional[str]

    company_identification_code: Optional[str]

    lang: LanguageCode

    resale: bool

    questions: Optional[List[ContactQuestion]]
    """
    :deprecated
    """

    extension_fr: Optional[ContactExtensionFR]

    extension_eu: Optional[ContactExtensionEU]

    whois_opt_in: bool

    state: Optional[str]

    extension_nl: Optional[ContactExtensionNL]


@dataclass
class OrderResponse:
    domains: List[str]

    organization_id: str

    project_id: str

    task_id: str

    created_at: Optional[datetime]


@dataclass
class RecordChange:
    add: Optional[RecordChangeAdd]
    """
    One-of ('change'): at most one of 'add', 'set_', 'delete', 'clear' could be set.
    """

    set_: Optional[RecordChangeSet]
    """
    One-of ('change'): at most one of 'add', 'set_', 'delete', 'clear' could be set.
    """

    delete: Optional[RecordChangeDelete]
    """
    One-of ('change'): at most one of 'add', 'set_', 'delete', 'clear' could be set.
    """

    clear: Optional[RecordChangeClear]
    """
    One-of ('change'): at most one of 'add', 'set_', 'delete', 'clear' could be set.
    """


@dataclass
class RecordChangeAdd:
    records: List[DomainRecord]


@dataclass
class RecordChangeClear:
    pass


@dataclass
class RecordChangeDelete:
    id: Optional[str]
    """
    One-of ('identifier'): at most one of 'id', 'id_fields' could be set.
    """

    id_fields: Optional[RecordIdentifier]
    """
    One-of ('identifier'): at most one of 'id', 'id_fields' could be set.
    """


@dataclass
class RecordChangeSet:
    id: Optional[str]
    """
    One-of ('identifier'): at most one of 'id', 'id_fields' could be set.
    """

    id_fields: Optional[RecordIdentifier]
    """
    One-of ('identifier'): at most one of 'id', 'id_fields' could be set.
    """

    records: List[DomainRecord]


@dataclass
class RecordIdentifier:
    name: str

    type_: DomainRecordType

    data: Optional[str]

    ttl: Optional[int]


@dataclass
class RefreshDNSZoneResponse:
    """
    Refresh dns zone response.
    """

    dns_zones: List[DNSZone]
    """
    DNS zones returned.
    """


@dataclass
class RegisterExternalDomainResponse:
    domain: str

    organization_id: str

    validation_token: str

    created_at: Optional[datetime]

    project_id: str


@dataclass
class RenewableDomain:
    domain: str

    project_id: str

    organization_id: str

    status: RenewableDomainStatus

    renewable_duration_in_years: Optional[int]

    expired_at: Optional[datetime]

    limit_renew_at: Optional[datetime]

    limit_redemption_at: Optional[datetime]

    estimated_delete_at: Optional[datetime]

    tld: Optional[Tld]


@dataclass
class RestoreDNSZoneVersionResponse:
    """
    Restore dns zone version response.
    """


@dataclass
class SSLCertificate:
    dns_zone: str

    alternative_dns_zones: List[str]

    status: SSLCertificateStatus

    private_key: str

    certificate_chain: str

    created_at: Optional[datetime]

    expired_at: Optional[datetime]


@dataclass
class SearchAvailableDomainsResponse:
    """
    Search available domains response.
    """

    available_domains: List[AvailableDomain]
    """
    Array of available domains.
    """


@dataclass
class Task:
    id: str

    project_id: str

    organization_id: str

    domain: Optional[str]

    type_: TaskType

    status: TaskStatus

    started_at: Optional[datetime]

    updated_at: Optional[datetime]

    message: Optional[str]


@dataclass
class Tld:
    name: str

    dnssec_support: bool

    duration_in_years_min: int

    duration_in_years_max: int

    idn_support: bool

    offers: Dict[str, TldOffer]

    specifications: Dict[str, str]


@dataclass
class TldOffer:
    action: str

    operation_path: str

    price: Optional[Money]


@dataclass
class TransferInDomainRequestTransferRequest:
    domain: str

    auth_code: str


@dataclass
class UpdateContactRequestQuestion:
    question: Optional[str]

    answer: Optional[str]


@dataclass
class UpdateDNSZoneNameserversResponse:
    """
    Update dns zone nameservers response.
    """

    ns: List[Nameserver]
    """
    DNS zone name servers returned.
    """


@dataclass
class UpdateDNSZoneRecordsResponse:
    """
    Update dns zone records response.
    """

    records: List[DomainRecord]
    """
    DNS zone records returned.
    """


@dataclass
class ListDNSZonesRequest:
    organization_id: Optional[str]
    """
    Organization ID on which to filter the returned DNS zones.
    """

    project_id: Optional[str]
    """
    Project ID on which to filter the returned DNS zones.
    """

    order_by: Optional[ListDNSZonesRequestOrderBy]
    """
    Sort order of the returned DNS zones.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of DNS zones to return per page.
    """

    domain: str
    """
    Domain on which to filter the returned DNS zones.
    """

    dns_zone: str
    """
    DNS zone on which to filter the returned DNS zones.
    """


@dataclass
class CreateDNSZoneRequest:
    domain: str
    """
    Domain in which to crreate the DNS zone.
    """

    subdomain: str
    """
    Subdomain of the DNS zone to create.
    """

    project_id: Optional[str]
    """
    Project ID in which to create the DNS zone.
    """


@dataclass
class UpdateDNSZoneRequest:
    dns_zone: str
    """
    DNS zone to update.
    """

    new_dns_zone: str
    """
    Name of the new DNS zone to create.
    """

    project_id: Optional[str]
    """
    Project ID in which to create the new DNS zone.
    """


@dataclass
class CloneDNSZoneRequest:
    dns_zone: str
    """
    DNS zone to clone.
    """

    dest_dns_zone: str
    """
    Destination DNS zone in which to clone the chosen DNS zone.
    """

    overwrite: bool
    """
    Specifies whether or not the destination DNS zone will be overwritten.
    """

    project_id: Optional[str]
    """
    Project ID of the destination DNS zone.
    """


@dataclass
class DeleteDNSZoneRequest:
    dns_zone: str
    """
    DNS zone to delete.
    """

    project_id: Optional[str]
    """
    Project ID of the DNS zone to delete.
    """


@dataclass
class ListDNSZoneRecordsRequest:
    dns_zone: str
    """
    DNS zone on which to filter the returned DNS zone records.
    """

    project_id: Optional[str]
    """
    Project ID on which to filter the returned DNS zone records.
    """

    order_by: Optional[ListDNSZoneRecordsRequestOrderBy]
    """
    Sort order of the returned DNS zone records.
    """

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of DNS zone records per page.
    """

    name: str
    """
    Name on which to filter the returned DNS zone records.
    """

    type_: Optional[DomainRecordType]
    """
    Record type on which to filter the returned DNS zone records.
    """

    id: Optional[str]
    """
    Record ID on which to filter the returned DNS zone records.
    """


@dataclass
class UpdateDNSZoneRecordsRequest:
    dns_zone: str
    """
    DNS zone in which to update the DNS zone records.
    """

    changes: List[RecordChange]
    """
    Changes made to the records.
    """

    return_all_records: Optional[bool]
    """
    Specifies whether or not to return all the records.
    """

    disallow_new_zone_creation: bool
    """
    Disable the creation of the target zone if it does not exist. Target zone creation is disabled by default.
    """

    serial: Optional[int]
    """
    Use the provided serial (0) instead of the auto-increment serial.
    """


@dataclass
class ListDNSZoneNameserversRequest:
    dns_zone: str
    """
    DNS zone on which to filter the returned DNS zone name servers.
    """

    project_id: Optional[str]
    """
    Project ID on which to filter the returned DNS zone name servers.
    """


@dataclass
class UpdateDNSZoneNameserversRequest:
    dns_zone: str
    """
    DNS zone in which to update the DNS zone name servers.
    """

    ns: List[Nameserver]
    """
    New DNS zone name servers.
    """


@dataclass
class ClearDNSZoneRecordsRequest:
    dns_zone: str
    """
    DNS zone to clear.
    """


@dataclass
class ExportRawDNSZoneRequest:
    dns_zone: str
    """
    DNS zone to export.
    """

    format: RawFormat
    """
    DNS zone format.
    """


@dataclass
class ImportRawDNSZoneRequest:
    dns_zone: str
    """
    DNS zone to import.
    """

    content: Optional[str]
    """
    :deprecated
    """

    project_id: Optional[str]

    format: Optional[RawFormat]
    """
    :deprecated
    """

    bind_source: Optional[ImportRawDNSZoneRequestBindSource]
    """
    Import a bind file format.
    
    One-of ('source'): at most one of 'bind_source', 'axfr_source' could be set.
    """

    axfr_source: Optional[ImportRawDNSZoneRequestAXFRSource]
    """
    Import from the name server given with TSIG, to use or not.
    
    One-of ('source'): at most one of 'bind_source', 'axfr_source' could be set.
    """


@dataclass
class ImportProviderDNSZoneRequest:
    dns_zone: str

    online_v1: Optional[ImportProviderDNSZoneRequestOnlineV1]
    """
    One-of ('provider'): at most one of 'online_v1' could be set.
    """


@dataclass
class RefreshDNSZoneRequest:
    dns_zone: str
    """
    DNS zone to refresh.
    """

    recreate_dns_zone: bool
    """
    Specifies whether or not to recreate the DNS zone.
    """

    recreate_sub_dns_zone: bool
    """
    Specifies whether or not to recreate the sub DNS zone.
    """


@dataclass
class ListDNSZoneVersionsRequest:
    dns_zone: str

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of DNS zones versions per page.
    """


@dataclass
class ListDNSZoneVersionRecordsRequest:
    dns_zone_version_id: str

    page: Optional[int]
    """
    Page number to return, from the paginated results.
    """

    page_size: Optional[int]
    """
    Maximum number of DNS zones versions records per page.
    """


@dataclass
class GetDNSZoneVersionDiffRequest:
    dns_zone_version_id: str


@dataclass
class RestoreDNSZoneVersionRequest:
    dns_zone_version_id: str


@dataclass
class GetSSLCertificateRequest:
    dns_zone: str


@dataclass
class CreateSSLCertificateRequest:
    dns_zone: str

    alternative_dns_zones: Optional[List[str]]


@dataclass
class ListSSLCertificatesRequest:
    dns_zone: str

    page: Optional[int]

    page_size: Optional[int]

    project_id: Optional[str]


@dataclass
class DeleteSSLCertificateRequest:
    dns_zone: str


@dataclass
class GetDNSZoneTsigKeyRequest:
    dns_zone: str


@dataclass
class DeleteDNSZoneTsigKeyRequest:
    dns_zone: str


@dataclass
class RegistrarApiListTasksRequest:
    page: Optional[int]

    page_size: Optional[int]

    project_id: Optional[str]

    organization_id: Optional[str]

    domain: Optional[str]

    types: Optional[List[TaskType]]

    statuses: Optional[List[TaskStatus]]

    order_by: Optional[ListTasksRequestOrderBy]


@dataclass
class RegistrarApiBuyDomainsRequest:
    domains: List[str]

    duration_in_years: int

    project_id: Optional[str]

    owner_contact_id: Optional[str]
    """
    One-of ('owner_contact_type'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    owner_contact: Optional[NewContact]
    """
    One-of ('owner_contact_type'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    administrative_contact_id: Optional[str]
    """
    One-of ('administrative_contact_type'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    administrative_contact: Optional[NewContact]
    """
    One-of ('administrative_contact_type'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    technical_contact_id: Optional[str]
    """
    One-of ('technical_contact_type'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """

    technical_contact: Optional[NewContact]
    """
    One-of ('technical_contact_type'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """


@dataclass
class RegistrarApiRenewDomainsRequest:
    domains: List[str]

    duration_in_years: int

    force_late_renewal: Optional[bool]


@dataclass
class RegistrarApiTransferInDomainRequest:
    domains: List[TransferInDomainRequestTransferRequest]

    project_id: Optional[str]

    owner_contact_id: Optional[str]
    """
    One-of ('owner_contact_type'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    owner_contact: Optional[NewContact]
    """
    One-of ('owner_contact_type'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    administrative_contact_id: Optional[str]
    """
    One-of ('administrative_contact_type'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    administrative_contact: Optional[NewContact]
    """
    One-of ('administrative_contact_type'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    technical_contact_id: Optional[str]
    """
    One-of ('technical_contact_type'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """

    technical_contact: Optional[NewContact]
    """
    One-of ('technical_contact_type'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """


@dataclass
class RegistrarApiTradeDomainRequest:
    domain: str

    project_id: Optional[str]

    new_owner_contact_id: Optional[str]
    """
    One-of ('new_owner_contact_type'): at most one of 'new_owner_contact_id', 'new_owner_contact' could be set.
    """

    new_owner_contact: Optional[NewContact]
    """
    One-of ('new_owner_contact_type'): at most one of 'new_owner_contact_id', 'new_owner_contact' could be set.
    """


@dataclass
class RegistrarApiRegisterExternalDomainRequest:
    domain: str

    project_id: Optional[str]


@dataclass
class RegistrarApiDeleteExternalDomainRequest:
    domain: str


@dataclass
class RegistrarApiCheckContactsCompatibilityRequest:
    domains: Optional[List[str]]

    tlds: Optional[List[str]]

    owner_contact_id: Optional[str]
    """
    One-of ('owner_contact_type'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    owner_contact: Optional[NewContact]
    """
    One-of ('owner_contact_type'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    administrative_contact_id: Optional[str]
    """
    One-of ('administrative_contact_type'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    administrative_contact: Optional[NewContact]
    """
    One-of ('administrative_contact_type'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    technical_contact_id: Optional[str]
    """
    One-of ('technical_contact_type'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """

    technical_contact: Optional[NewContact]
    """
    One-of ('technical_contact_type'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """


@dataclass
class RegistrarApiListContactsRequest:
    page: Optional[int]

    page_size: Optional[int]

    domain: Optional[str]

    project_id: Optional[str]

    organization_id: Optional[str]


@dataclass
class RegistrarApiGetContactRequest:
    contact_id: str


@dataclass
class RegistrarApiUpdateContactRequest:
    contact_id: str

    email: Optional[str]

    email_alt: Optional[str]

    phone_number: Optional[str]

    fax_number: Optional[str]

    address_line_1: Optional[str]

    address_line_2: Optional[str]

    zip: Optional[str]

    city: Optional[str]

    country: Optional[str]

    vat_identification_code: Optional[str]

    company_identification_code: Optional[str]

    lang: LanguageCode

    resale: Optional[bool]

    questions: Optional[List[UpdateContactRequestQuestion]]
    """
    :deprecated
    """

    extension_fr: Optional[ContactExtensionFR]

    extension_eu: Optional[ContactExtensionEU]

    whois_opt_in: Optional[bool]

    state: Optional[str]

    extension_nl: Optional[ContactExtensionNL]


@dataclass
class RegistrarApiListDomainsRequest:
    page: Optional[int]

    page_size: Optional[int]

    order_by: Optional[ListDomainsRequestOrderBy]

    registrar: Optional[str]

    status: Optional[DomainStatus]

    project_id: Optional[str]

    organization_id: Optional[str]

    is_external: Optional[bool]

    domain: Optional[str]


@dataclass
class RegistrarApiListRenewableDomainsRequest:
    page: Optional[int]

    page_size: Optional[int]

    order_by: Optional[ListRenewableDomainsRequestOrderBy]

    project_id: Optional[str]

    organization_id: Optional[str]


@dataclass
class RegistrarApiGetDomainRequest:
    domain: str


@dataclass
class RegistrarApiUpdateDomainRequest:
    domain: str

    technical_contact_id: Optional[str]
    """
    One-of ('technical_contact_info'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """

    technical_contact: Optional[NewContact]
    """
    One-of ('technical_contact_info'): at most one of 'technical_contact_id', 'technical_contact' could be set.
    """

    owner_contact_id: Optional[str]
    """
    One-of ('owner_contact_info'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    owner_contact: Optional[NewContact]
    """
    One-of ('owner_contact_info'): at most one of 'owner_contact_id', 'owner_contact' could be set.
    """

    administrative_contact_id: Optional[str]
    """
    One-of ('administrative_contact_info'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """

    administrative_contact: Optional[NewContact]
    """
    One-of ('administrative_contact_info'): at most one of 'administrative_contact_id', 'administrative_contact' could be set.
    """


@dataclass
class RegistrarApiLockDomainTransferRequest:
    domain: str


@dataclass
class RegistrarApiUnlockDomainTransferRequest:
    domain: str


@dataclass
class RegistrarApiEnableDomainAutoRenewRequest:
    domain: str


@dataclass
class RegistrarApiDisableDomainAutoRenewRequest:
    domain: str


@dataclass
class RegistrarApiGetDomainAuthCodeRequest:
    domain: str


@dataclass
class RegistrarApiEnableDomainDNSSECRequest:
    domain: str

    ds_record: Optional[DSRecord]


@dataclass
class RegistrarApiDisableDomainDNSSECRequest:
    domain: str


@dataclass
class RegistrarApiSearchAvailableDomainsRequest:
    domains: List[str]
    """
    A list of domain to search, TLD is optional.
    """

    tlds: Optional[List[str]]
    """
    Array of tlds to search on.
    """

    strict_search: bool
    """
    Search exact match.
    """


@dataclass
class RegistrarApiCreateDomainHostRequest:
    domain: str

    name: str

    ips: Optional[List[str]]


@dataclass
class RegistrarApiListDomainHostsRequest:
    domain: str

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class RegistrarApiUpdateDomainHostRequest:
    domain: str

    name: str

    ips: Optional[List[str]]


@dataclass
class RegistrarApiDeleteDomainHostRequest:
    domain: str

    name: str



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    Product,
    AccessSecretVersionResponse,
    ListSecretVersionsResponse,
    ListSecretsResponse,
    PasswordGenerationParams,
    Secret,
    SecretVersion,
    CreateSecretRequest,
    UpdateSecretRequest,
    AddSecretOwnerRequest,
    CreateSecretVersionRequest,
    GeneratePasswordRequest,
    UpdateSecretVersionRequest,
)


def unmarshal_Secret(data: Any) -> Secret:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Secret' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description")
    args["description"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("is_managed")
    args["is_managed"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("region")
    args["region"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("tags")
    args["tags"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("version_count")
    args["version_count"] = field

    return Secret(**args)


def unmarshal_SecretVersion(data: Any) -> SecretVersion:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SecretVersion' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("created_at")
    args["created_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("description")
    args["description"] = field

    field = data.get("is_latest")
    args["is_latest"] = field

    field = data.get("revision")
    args["revision"] = field

    field = data.get("secret_id")
    args["secret_id"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("updated_at")
    args["updated_at"] = parser.isoparse(field) if type(field) is str else field

    return SecretVersion(**args)


def unmarshal_AccessSecretVersionResponse(data: Any) -> AccessSecretVersionResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'AccessSecretVersionResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("data")
    args["data"] = field

    field = data.get("data_crc32")
    args["data_crc32"] = field

    field = data.get("revision")
    args["revision"] = field

    field = data.get("secret_id")
    args["secret_id"] = field

    return AccessSecretVersionResponse(**args)


def unmarshal_ListSecretVersionsResponse(data: Any) -> ListSecretVersionsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListSecretVersionsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count")
    args["total_count"] = field

    field = data.get("versions")
    args["versions"] = [unmarshal_SecretVersion(v) for v in data["versions"]]

    return ListSecretVersionsResponse(**args)


def unmarshal_ListSecretsResponse(data: Any) -> ListSecretsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListSecretsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("secrets")
    args["secrets"] = [unmarshal_Secret(v) for v in data["secrets"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListSecretsResponse(**args)


def marshal_PasswordGenerationParams(
    request: PasswordGenerationParams,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "additional_chars": request.additional_chars,
        "length": request.length,
        "no_digits": request.no_digits,
        "no_lowercase_letters": request.no_lowercase_letters,
        "no_uppercase_letters": request.no_uppercase_letters,
    }


def marshal_AddSecretOwnerRequest(
    request: AddSecretOwnerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "product": Product(request.product),
        "product_name": request.product_name,
    }


def marshal_CreateSecretRequest(
    request: CreateSecretRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "name": request.name,
        "project_id": request.project_id or defaults.default_project_id,
        "tags": request.tags,
    }


def marshal_CreateSecretVersionRequest(
    request: CreateSecretVersionRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        **resolve_one_of(
            [
                OneOfPossibility(
                    "password_generation",
                    marshal_PasswordGenerationParams(
                        request.password_generation, defaults
                    )
                    if request.password_generation is not None
                    else None,
                ),
            ]
        ),
        "data": request.data,
        "data_crc32": request.data_crc32,
        "description": request.description,
        "disable_previous": request.disable_previous,
    }


def marshal_GeneratePasswordRequest(
    request: GeneratePasswordRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "additional_chars": request.additional_chars,
        "description": request.description,
        "disable_previous": request.disable_previous,
        "length": request.length,
        "no_digits": request.no_digits,
        "no_lowercase_letters": request.no_lowercase_letters,
        "no_uppercase_letters": request.no_uppercase_letters,
    }


def marshal_UpdateSecretRequest(
    request: UpdateSecretRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "name": request.name,
        "tags": request.tags,
    }


def marshal_UpdateSecretVersionRequest(
    request: UpdateSecretVersionRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
    }



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from dateutil import parser
from .types import (
    ContainerHttpOption,
    ContainerPrivacy,
    ContainerProtocol,
    Container,
    Cron,
    Domain,
    ListContainersResponse,
    ListCronsResponse,
    ListDomainsResponse,
    ListLogsResponse,
    ListNamespacesResponse,
    ListTokensResponse,
    Log,
    Namespace,
    Secret,
    SecretHashedValue,
    Token,
    CreateNamespaceRequest,
    UpdateNamespaceRequest,
    CreateContainerRequest,
    UpdateContainerRequest,
    CreateCronRequest,
    UpdateCronRequest,
    CreateDomainRequest,
    CreateTokenRequest,
)


def unmarshal_SecretHashedValue(data: Any) -> SecretHashedValue:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'SecretHashedValue' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("hashed_value")
    args["hashed_value"] = field

    field = data.get("key")
    args["key"] = field

    return SecretHashedValue(**args)


def unmarshal_Container(data: Any) -> Container:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Container' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("cpu_limit")
    args["cpu_limit"] = field

    field = data.get("description")
    args["description"] = field

    field = data.get("domain_name")
    args["domain_name"] = field

    field = data.get("environment_variables")
    args["environment_variables"] = field

    field = data.get("error_message")
    args["error_message"] = field

    field = data.get("http_option")
    args["http_option"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("max_concurrency")
    args["max_concurrency"] = field

    field = data.get("max_scale")
    args["max_scale"] = field

    field = data.get("memory_limit")
    args["memory_limit"] = field

    field = data.get("min_scale")
    args["min_scale"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("namespace_id")
    args["namespace_id"] = field

    field = data.get("port")
    args["port"] = field

    field = data.get("privacy")
    args["privacy"] = field

    field = data.get("protocol")
    args["protocol"] = field

    field = data.get("region")
    args["region"] = field

    field = data.get("registry_image")
    args["registry_image"] = field

    field = data.get("secret_environment_variables")
    args["secret_environment_variables"] = [
        unmarshal_SecretHashedValue(v) for v in data["secret_environment_variables"]
    ]

    field = data.get("status")
    args["status"] = field

    field = data.get("timeout")
    args["timeout"] = field

    return Container(**args)


def unmarshal_Cron(data: Any) -> Cron:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Cron' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("args")
    args["args"] = field

    field = data.get("container_id")
    args["container_id"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("schedule")
    args["schedule"] = field

    field = data.get("status")
    args["status"] = field

    return Cron(**args)


def unmarshal_Domain(data: Any) -> Domain:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Domain' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("container_id")
    args["container_id"] = field

    field = data.get("error_message")
    args["error_message"] = field

    field = data.get("hostname")
    args["hostname"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("url")
    args["url"] = field

    return Domain(**args)


def unmarshal_Log(data: Any) -> Log:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Log' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id")
    args["id"] = field

    field = data.get("level")
    args["level"] = field

    field = data.get("message")
    args["message"] = field

    field = data.get("source")
    args["source"] = field

    field = data.get("stream")
    args["stream"] = field

    field = data.get("timestamp")
    args["timestamp"] = parser.isoparse(field) if type(field) is str else field

    return Log(**args)


def unmarshal_Namespace(data: Any) -> Namespace:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Namespace' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("description")
    args["description"] = field

    field = data.get("environment_variables")
    args["environment_variables"] = field

    field = data.get("error_message")
    args["error_message"] = field

    field = data.get("id")
    args["id"] = field

    field = data.get("name")
    args["name"] = field

    field = data.get("organization_id")
    args["organization_id"] = field

    field = data.get("project_id")
    args["project_id"] = field

    field = data.get("region")
    args["region"] = field

    field = data.get("registry_endpoint")
    args["registry_endpoint"] = field

    field = data.get("registry_namespace_id")
    args["registry_namespace_id"] = field

    field = data.get("secret_environment_variables")
    args["secret_environment_variables"] = [
        unmarshal_SecretHashedValue(v) for v in data["secret_environment_variables"]
    ]

    field = data.get("status")
    args["status"] = field

    return Namespace(**args)


def unmarshal_Token(data: Any) -> Token:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'Token' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("container_id")
    args["container_id"] = field

    field = data.get("description")
    args["description"] = field

    field = data.get("expires_at")
    args["expires_at"] = parser.isoparse(field) if type(field) is str else field

    field = data.get("id")
    args["id"] = field

    field = data.get("namespace_id")
    args["namespace_id"] = field

    field = data.get("public_key")
    args["public_key"] = field

    field = data.get("status")
    args["status"] = field

    field = data.get("token")
    args["token"] = field

    return Token(**args)


def unmarshal_ListContainersResponse(data: Any) -> ListContainersResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListContainersResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("containers")
    args["containers"] = [unmarshal_Container(v) for v in data["containers"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListContainersResponse(**args)


def unmarshal_ListCronsResponse(data: Any) -> ListCronsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListCronsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("crons")
    args["crons"] = [unmarshal_Cron(v) for v in data["crons"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListCronsResponse(**args)


def unmarshal_ListDomainsResponse(data: Any) -> ListDomainsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListDomainsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("domains")
    args["domains"] = [unmarshal_Domain(v) for v in data["domains"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListDomainsResponse(**args)


def unmarshal_ListLogsResponse(data: Any) -> ListLogsResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListLogsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("logs")
    args["logs"] = [unmarshal_Log(v) for v in data["logs"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListLogsResponse(**args)


def unmarshal_ListNamespacesResponse(data: Any) -> ListNamespacesResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListNamespacesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("namespaces")
    args["namespaces"] = [unmarshal_Namespace(v) for v in data["namespaces"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListNamespacesResponse(**args)


def unmarshal_ListTokensResponse(data: Any) -> ListTokensResponse:
    if type(data) is not dict:
        raise TypeError(
            f"Unmarshalling the type 'ListTokensResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("tokens")
    args["tokens"] = [unmarshal_Token(v) for v in data["tokens"]]

    field = data.get("total_count")
    args["total_count"] = field

    return ListTokensResponse(**args)


def marshal_Secret(
    request: Secret,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "key": request.key,
        "value": request.value,
    }


def marshal_CreateContainerRequest(
    request: CreateContainerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "cpu_limit": request.cpu_limit,
        "description": request.description,
        "environment_variables": request.environment_variables,
        "http_option": ContainerHttpOption(request.http_option),
        "max_concurrency": request.max_concurrency,
        "max_scale": request.max_scale,
        "memory_limit": request.memory_limit,
        "min_scale": request.min_scale,
        "name": request.name,
        "namespace_id": request.namespace_id,
        "port": request.port,
        "privacy": ContainerPrivacy(request.privacy),
        "protocol": ContainerProtocol(request.protocol),
        "registry_image": request.registry_image,
        "secret_environment_variables": [
            marshal_Secret(v, defaults) for v in request.secret_environment_variables
        ]
        if request.secret_environment_variables is not None
        else None,
        "timeout": request.timeout,
    }


def marshal_CreateCronRequest(
    request: CreateCronRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "args": request.args,
        "container_id": request.container_id,
        "name": request.name,
        "schedule": request.schedule,
    }


def marshal_CreateDomainRequest(
    request: CreateDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "container_id": request.container_id,
        "hostname": request.hostname,
    }


def marshal_CreateNamespaceRequest(
    request: CreateNamespaceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "environment_variables": request.environment_variables,
        "name": request.name,
        "project_id": request.project_id or defaults.default_project_id,
        "secret_environment_variables": [
            marshal_Secret(v, defaults) for v in request.secret_environment_variables
        ]
        if request.secret_environment_variables is not None
        else None,
    }


def marshal_CreateTokenRequest(
    request: CreateTokenRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        **resolve_one_of(
            [
                OneOfPossibility(
                    "container_id",
                    request.container_id if request.container_id is not None else None,
                ),
                OneOfPossibility(
                    "namespace_id",
                    request.namespace_id if request.namespace_id is not None else None,
                ),
            ]
        ),
        "description": request.description,
        "expires_at": request.expires_at,
    }


def marshal_UpdateContainerRequest(
    request: UpdateContainerRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "cpu_limit": request.cpu_limit,
        "description": request.description,
        "environment_variables": request.environment_variables,
        "http_option": ContainerHttpOption(request.http_option),
        "max_concurrency": request.max_concurrency,
        "max_scale": request.max_scale,
        "memory_limit": request.memory_limit,
        "min_scale": request.min_scale,
        "port": request.port,
        "privacy": ContainerPrivacy(request.privacy),
        "protocol": ContainerProtocol(request.protocol),
        "redeploy": request.redeploy,
        "registry_image": request.registry_image,
        "secret_environment_variables": [
            marshal_Secret(v, defaults) for v in request.secret_environment_variables
        ]
        if request.secret_environment_variables is not None
        else None,
        "timeout": request.timeout,
    }


def marshal_UpdateCronRequest(
    request: UpdateCronRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "args": request.args,
        "container_id": request.container_id,
        "name": request.name,
        "schedule": request.schedule,
    }


def marshal_UpdateNamespaceRequest(
    request: UpdateNamespaceRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    return {
        "description": request.description,
        "environment_variables": request.environment_variables,
        "secret_environment_variables": [
            marshal_Secret(v, defaults) for v in request.secret_environment_variables
        ]
        if request.secret_environment_variables is not None
        else None,
    }



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region,
)
from scaleway_core.utils import (
    WaitForOptions,
    fetch_all_pages,
    random_name,
    validate_path_param,
    wait_for_resource,
)
from .types import (
    ImageVisibility,
    ListImagesRequestOrderBy,
    ListNamespacesRequestOrderBy,
    ListTagsRequestOrderBy,
    Image,
    ListImagesResponse,
    ListNamespacesResponse,
    ListTagsResponse,
    Namespace,
    Tag,
    CreateNamespaceRequest,
    UpdateNamespaceRequest,
    UpdateImageRequest,
)
from .content import (
    IMAGE_TRANSIENT_STATUSES,
    NAMESPACE_TRANSIENT_STATUSES,
    TAG_TRANSIENT_STATUSES,
)
from .marshalling import (
    marshal_CreateNamespaceRequest,
    marshal_UpdateImageRequest,
    marshal_UpdateNamespaceRequest,
    unmarshal_Image,
    unmarshal_Namespace,
    unmarshal_Tag,
    unmarshal_ListImagesResponse,
    unmarshal_ListNamespacesResponse,
    unmarshal_ListTagsResponse,
)


class RegistryV1API(API):
    """
    Container Registry API.

    Container Registry API.
    """

    def list_namespaces(
        self,
        *,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: ListNamespacesRequestOrderBy = ListNamespacesRequestOrderBy.CREATED_AT_ASC,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
    ) -> ListNamespacesResponse:
        """
        List namespaces.
        List all namespaces in a specified region. By default, the namespaces listed are ordered by creation date in ascending order. This can be modified via the order_by field. You can also define additional parameters for your query, such as the `instance_id` and `project_id` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: A positive integer to choose the page to display.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param order_by: Criteria to use when ordering namespace listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
        :param organization_id: Filter by Organization ID.
        :param project_id: Filter by Project ID.
        :param name: Filter by the namespace name (exact match).
        :return: :class:`ListNamespacesResponse <ListNamespacesResponse>`

        Usage:
        ::

            result = api.list_namespaces()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/registry/v1/regions/{param_region}/namespaces",
            params={
                "name": name,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListNamespacesResponse(res.json())

    def list_namespaces_all(
        self,
        *,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListNamespacesRequestOrderBy] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
        name: Optional[str] = None,
    ) -> List[Namespace]:
        """
        List namespaces.
        List all namespaces in a specified region. By default, the namespaces listed are ordered by creation date in ascending order. This can be modified via the order_by field. You can also define additional parameters for your query, such as the `instance_id` and `project_id` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: A positive integer to choose the page to display.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param order_by: Criteria to use when ordering namespace listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
        :param organization_id: Filter by Organization ID.
        :param project_id: Filter by Project ID.
        :param name: Filter by the namespace name (exact match).
        :return: :class:`List[ListNamespacesResponse] <List[ListNamespacesResponse]>`

        Usage:
        ::

            result = api.list_namespaces_all()
        """

        return fetch_all_pages(
            type=ListNamespacesResponse,
            key="namespaces",
            fetcher=self.list_namespaces,
            args={
                "region": region,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "organization_id": organization_id,
                "project_id": project_id,
                "name": name,
            },
        )

    def get_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
    ) -> Namespace:
        """
        Get a namespace.
        Retrieve information about a given namespace, specified by its `namespace_id` and region. Full details about the namespace, such as `description`, `project_id`, `status`, `endpoint`, `is_public`, `size`, and `image_count` are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: UUID of the namespace.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = api.get_namespace(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_namespace_id = validate_path_param("namespace_id", namespace_id)

        res = self._request(
            "GET",
            f"/registry/v1/regions/{param_region}/namespaces/{param_namespace_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    def wait_for_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Namespace, bool]] = None,
    ) -> Namespace:
        """
        Waits for :class:`Namespace <Namespace>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: UUID of the namespace.
        :param options: The options for the waiter
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = api.wait_for_namespace(namespace_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in NAMESPACE_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_namespace,
            options=options,
            args={
                "namespace_id": namespace_id,
                "region": region,
            },
        )

    def create_namespace(
        self,
        *,
        description: str,
        is_public: bool,
        region: Optional[Region] = None,
        name: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
    ) -> Namespace:
        """
        Create a namespace.
        Create a new Container Registry namespace. You must specify the namespace name and region in which you want it to be created. Optionally, you can specify the `project_id` and `is_public` in the request payload.
        :param region: Region to target. If none is passed will use default region from the config.
        :param name: Name of the namespace.
        :param description: Description of the namespace.
        :param organization_id: Namespace owner (deprecated).

        One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
        :param project_id: Project ID on which the namespace will be created.

        One-of ('project_identifier'): at most one of 'organization_id', 'project_id' could be set.
        :param is_public: Defines whether or not namespace is public.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = api.create_namespace(
                description="example",
                is_public=True,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/registry/v1/regions/{param_region}/namespaces",
            body=marshal_CreateNamespaceRequest(
                CreateNamespaceRequest(
                    description=description,
                    is_public=is_public,
                    region=region,
                    name=name or random_name(prefix="ns"),
                    organization_id=organization_id,
                    project_id=project_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    def update_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
        description: Optional[str] = None,
        is_public: Optional[bool] = None,
    ) -> Namespace:
        """
        Update a namespace.
        Update the parameters of a given namespace, specified by its `namespace_id` and `region`. You can update the `description` and `is_public` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: ID of the namespace to update.
        :param description: Namespace description.
        :param is_public: Defines whether or not the namespace is public.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = api.update_namespace(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_namespace_id = validate_path_param("namespace_id", namespace_id)

        res = self._request(
            "PATCH",
            f"/registry/v1/regions/{param_region}/namespaces/{param_namespace_id}",
            body=marshal_UpdateNamespaceRequest(
                UpdateNamespaceRequest(
                    namespace_id=namespace_id,
                    region=region,
                    description=description,
                    is_public=is_public,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    def delete_namespace(
        self,
        *,
        namespace_id: str,
        region: Optional[Region] = None,
    ) -> Namespace:
        """
        Delete a namespace.
        Delete a given namespace. You must specify, in the endpoint, the `region` and `namespace_id` parameters of the namespace you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param namespace_id: UUID of the namespace.
        :return: :class:`Namespace <Namespace>`

        Usage:
        ::

            result = api.delete_namespace(namespace_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_namespace_id = validate_path_param("namespace_id", namespace_id)

        res = self._request(
            "DELETE",
            f"/registry/v1/regions/{param_region}/namespaces/{param_namespace_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Namespace(res.json())

    def list_images(
        self,
        *,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: ListImagesRequestOrderBy = ListImagesRequestOrderBy.CREATED_AT_ASC,
        namespace_id: Optional[str] = None,
        name: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
    ) -> ListImagesResponse:
        """
        List images.
        List all images in a specified region. By default, the images listed are ordered by creation date in ascending order. This can be modified via the order_by field. You can also define additional parameters for your query, such as the `namespace_id` and `project_id` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: A positive integer to choose the page to display.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param order_by: Criteria to use when ordering image listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
        :param namespace_id: Filter by the namespace ID.
        :param name: Filter by the image name (exact match).
        :param organization_id: Filter by Organization ID.
        :param project_id: Filter by Project ID.
        :return: :class:`ListImagesResponse <ListImagesResponse>`

        Usage:
        ::

            result = api.list_images()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/registry/v1/regions/{param_region}/images",
            params={
                "name": name,
                "namespace_id": namespace_id,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListImagesResponse(res.json())

    def list_images_all(
        self,
        *,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListImagesRequestOrderBy] = None,
        namespace_id: Optional[str] = None,
        name: Optional[str] = None,
        organization_id: Optional[str] = None,
        project_id: Optional[str] = None,
    ) -> List[Image]:
        """
        List images.
        List all images in a specified region. By default, the images listed are ordered by creation date in ascending order. This can be modified via the order_by field. You can also define additional parameters for your query, such as the `namespace_id` and `project_id` parameters.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: A positive integer to choose the page to display.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param order_by: Criteria to use when ordering image listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
        :param namespace_id: Filter by the namespace ID.
        :param name: Filter by the image name (exact match).
        :param organization_id: Filter by Organization ID.
        :param project_id: Filter by Project ID.
        :return: :class:`List[ListImagesResponse] <List[ListImagesResponse]>`

        Usage:
        ::

            result = api.list_images_all()
        """

        return fetch_all_pages(
            type=ListImagesResponse,
            key="images",
            fetcher=self.list_images,
            args={
                "region": region,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "namespace_id": namespace_id,
                "name": name,
                "organization_id": organization_id,
                "project_id": project_id,
            },
        )

    def get_image(
        self,
        *,
        image_id: str,
        region: Optional[Region] = None,
    ) -> Image:
        """
        Get an image.
        Retrieve information about a given container image, specified by its `image_id` and region. Full details about the image, such as `name`, `namespace_id`, `status`, `visibility`, and `size` are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param image_id: UUID of the image.
        :return: :class:`Image <Image>`

        Usage:
        ::

            result = api.get_image(image_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "GET",
            f"/registry/v1/regions/{param_region}/images/{param_image_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Image(res.json())

    def wait_for_image(
        self,
        *,
        image_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Image, bool]] = None,
    ) -> Image:
        """
        Waits for :class:`Image <Image>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param image_id: UUID of the image.
        :param options: The options for the waiter
        :return: :class:`Image <Image>`

        Usage:
        ::

            result = api.wait_for_image(image_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in IMAGE_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_image,
            options=options,
            args={
                "image_id": image_id,
                "region": region,
            },
        )

    def update_image(
        self,
        *,
        image_id: str,
        visibility: ImageVisibility,
        region: Optional[Region] = None,
    ) -> Image:
        """
        Update an image.
        Update the parameters of a given image, specified by its `image_id` and `region`. You can update the `visibility` parameter.
        :param region: Region to target. If none is passed will use default region from the config.
        :param image_id: ID of the image to update.
        :param visibility: Set to `public` to allow the image to be pulled without authentication. Else, set to  `private`. Set to `inherit` to keep the same visibility configuration as the namespace.
        :return: :class:`Image <Image>`

        Usage:
        ::

            result = api.update_image(
                image_id="example",
                visibility=visibility_unknown,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "PATCH",
            f"/registry/v1/regions/{param_region}/images/{param_image_id}",
            body=marshal_UpdateImageRequest(
                UpdateImageRequest(
                    image_id=image_id,
                    visibility=visibility,
                    region=region,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Image(res.json())

    def delete_image(
        self,
        *,
        image_id: str,
        region: Optional[Region] = None,
    ) -> Image:
        """
        Delete an image.
        Delete a given image. You must specify, in the endpoint, the `region` and `image_id` parameters of the image you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param image_id: UUID of the image.
        :return: :class:`Image <Image>`

        Usage:
        ::

            result = api.delete_image(image_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "DELETE",
            f"/registry/v1/regions/{param_region}/images/{param_image_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Image(res.json())

    def list_tags(
        self,
        *,
        image_id: str,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: ListTagsRequestOrderBy = ListTagsRequestOrderBy.CREATED_AT_ASC,
        name: Optional[str] = None,
    ) -> ListTagsResponse:
        """
        List tags.
        List all tags for a given image, specified by region. By default, the tags listed are ordered by creation date in ascending order. This can be modified via the order_by field. You can also define additional parameters for your query, such as the `name`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param image_id: UUID of the image.
        :param page: A positive integer to choose the page to display.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param order_by: Criteria to use when ordering tag listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
        :param name: Filter by the tag name (exact match).
        :return: :class:`ListTagsResponse <ListTagsResponse>`

        Usage:
        ::

            result = api.list_tags(image_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_image_id = validate_path_param("image_id", image_id)

        res = self._request(
            "GET",
            f"/registry/v1/regions/{param_region}/images/{param_image_id}/tags",
            params={
                "name": name,
                "order_by": order_by,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListTagsResponse(res.json())

    def list_tags_all(
        self,
        *,
        image_id: str,
        region: Optional[Region] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListTagsRequestOrderBy] = None,
        name: Optional[str] = None,
    ) -> List[Tag]:
        """
        List tags.
        List all tags for a given image, specified by region. By default, the tags listed are ordered by creation date in ascending order. This can be modified via the order_by field. You can also define additional parameters for your query, such as the `name`.
        :param region: Region to target. If none is passed will use default region from the config.
        :param image_id: UUID of the image.
        :param page: A positive integer to choose the page to display.
        :param page_size: A positive integer lower or equal to 100 to select the number of items to display.
        :param order_by: Criteria to use when ordering tag listings. Possible values are `created_at_asc`, `created_at_desc`, `name_asc`, `name_desc`, `region`, `status_asc` and `status_desc`. The default value is `created_at_asc`.
        :param name: Filter by the tag name (exact match).
        :return: :class:`List[ListTagsResponse] <List[ListTagsResponse]>`

        Usage:
        ::

            result = api.list_tags_all(image_id="example")
        """

        return fetch_all_pages(
            type=ListTagsResponse,
            key="tags",
            fetcher=self.list_tags,
            args={
                "image_id": image_id,
                "region": region,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "name": name,
            },
        )

    def get_tag(
        self,
        *,
        tag_id: str,
        region: Optional[Region] = None,
    ) -> Tag:
        """
        Get a tag.
        Retrieve information about a given image tag, specified by its `tag_id` and region. Full details about the tag, such as `name`, `image_id`, `status`, and `digest` are returned in the response.
        :param region: Region to target. If none is passed will use default region from the config.
        :param tag_id: UUID of the tag.
        :return: :class:`Tag <Tag>`

        Usage:
        ::

            result = api.get_tag(tag_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_tag_id = validate_path_param("tag_id", tag_id)

        res = self._request(
            "GET",
            f"/registry/v1/regions/{param_region}/tags/{param_tag_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Tag(res.json())

    def wait_for_tag(
        self,
        *,
        tag_id: str,
        region: Optional[Region] = None,
        options: Optional[WaitForOptions[Tag, bool]] = None,
    ) -> Tag:
        """
        Waits for :class:`Tag <Tag>` to be in a final state.
        :param region: Region to target. If none is passed will use default region from the config.
        :param tag_id: UUID of the tag.
        :param options: The options for the waiter
        :return: :class:`Tag <Tag>`

        Usage:
        ::

            result = api.wait_for_tag(tag_id="example")
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in TAG_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_tag,
            options=options,
            args={
                "tag_id": tag_id,
                "region": region,
            },
        )

    def delete_tag(
        self,
        *,
        tag_id: str,
        region: Optional[Region] = None,
        force: Optional[bool] = None,
    ) -> Tag:
        """
        Delete a tag.
        Delete a given image tag. You must specify, in the endpoint, the `region` and `tag_id` parameters of the tag you want to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :param tag_id: UUID of the tag.
        :param force: If two tags share the same digest the deletion will fail unless this parameter is set to true (deprecated).
        :return: :class:`Tag <Tag>`

        Usage:
        ::

            result = api.delete_tag(tag_id="example")
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_tag_id = validate_path_param("tag_id", tag_id)

        res = self._request(
            "DELETE",
            f"/registry/v1/regions/{param_region}/tags/{param_tag_id}",
            params={
                "force": force,
            },
        )

        self._throw_on_error(res)
        return unmarshal_Tag(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    TimeSeries,
    Zone,
)


class AvailableClusterSettingPropertyType(str, Enum):
    UNKNOWN = "UNKNOWN"
    BOOLEAN = "BOOLEAN"
    INT = "INT"
    STRING = "STRING"

    def __str__(self) -> str:
        return str(self.value)


class ClusterStatus(str, Enum):
    UNKNOWN = "unknown"
    READY = "ready"
    PROVISIONING = "provisioning"
    CONFIGURING = "configuring"
    DELETING = "deleting"
    ERROR = "error"
    AUTOHEALING = "autohealing"
    LOCKED = "locked"
    SUSPENDED = "suspended"
    INITIALIZING = "initializing"

    def __str__(self) -> str:
        return str(self.value)


class ListClustersRequestOrderBy(str, Enum):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"

    def __str__(self) -> str:
        return str(self.value)


class NodeTypeStock(str, Enum):
    UNKNOWN = "unknown"
    LOW_STOCK = "low_stock"
    OUT_OF_STOCK = "out_of_stock"
    AVAILABLE = "available"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class ACLRule:
    """
    Acl rule.
    """

    id: str
    """
    ID of the rule.
    """

    ip_cidr: Optional[str]
    """
    IPv4 network address of the rule.
    """

    description: Optional[str]
    """
    Description of the rule.
    """


@dataclass
class ACLRuleSpec:
    """
    Acl rule spec.
    """

    ip_cidr: str
    """
    IPv4 network address of the rule.
    """

    description: str
    """
    Description of the rule.
    """


@dataclass
class AddAclRulesResponse:
    """
    Add acl rules response.
    """

    acl_rules: List[ACLRule]
    """
    ACL Rules enabled for the Database Instance.
    """

    total_count: int
    """
    Total count of ACL rules of the Database Instance.
    """


@dataclass
class AddEndpointsResponse:
    """
    Add endpoints response.
    """

    endpoints: List[Endpoint]
    """
    Endpoints defined on the Database Instance.
    """

    total_count: int
    """
    Total count of endpoints of the Database Instance.
    """


@dataclass
class AvailableClusterSetting:
    """
    Available cluster setting.
    """

    name: str
    """
    Name of the setting.
    """

    default_value: Optional[str]
    """
    Default value of the setting.
    """

    type_: AvailableClusterSettingPropertyType
    """
    Type of setting.
    """

    description: str
    """
    Description of the setting.
    """

    max_value: Optional[int]
    """
    Optional maximum value of the setting.
    """

    min_value: Optional[int]
    """
    Optional minimum value of the setting.
    """

    regex: Optional[str]
    """
    Optional validation rule of the setting.
    """

    deprecated: bool
    """
    Defines whether or not the setting is deprecated.
    """


@dataclass
class Cluster:
    """
    Cluster.
    """

    id: str
    """
    UUID of the Database Instance.
    """

    name: str
    """
    Name of the Database Instance.
    """

    project_id: str
    """
    Project ID the Database Instance belongs to.
    """

    status: ClusterStatus
    """
    Status of the Database Instance.
    """

    version: str
    """
    Redis™ engine version of the Database Instance.
    """

    endpoints: List[Endpoint]
    """
    List of Database Instance endpoints.
    """

    tags: List[str]
    """
    List of tags applied to the Database Instance.
    """

    node_type: str
    """
    Node type of the Database Instance.
    """

    created_at: Optional[datetime]
    """
    Creation date (Format ISO 8601).
    """

    updated_at: Optional[datetime]
    """
    Update date (Format ISO 8601).
    """

    tls_enabled: bool
    """
    Defines whether or not TLS is enabled.
    """

    cluster_settings: List[ClusterSetting]
    """
    List of Database Instance settings.
    """

    acl_rules: List[ACLRule]
    """
    List of ACL rules.
    """

    cluster_size: int
    """
    Number of nodes of the Database Instance cluster.
    """

    zone: Zone
    """
    Zone of the Database Instance.
    """

    user_name: str
    """
    Name of the user associated to the cluster.
    """

    upgradable_versions: List[str]
    """
    List of engine versions the Database Instance can upgrade to.
    """


@dataclass
class ClusterMetricsResponse:
    """
    Cluster metrics response.
    """

    timeseries: List[TimeSeries]
    """
    Time series of metrics of a given cluster.
    """


@dataclass
class ClusterSetting:
    """
    Cluster setting.
    """

    value: str
    """
    Value of the setting.
    """

    name: str
    """
    Name of the setting.
    """


@dataclass
class ClusterSettingsResponse:
    """
    Cluster settings response.
    """

    settings: List[ClusterSetting]
    """
    Settings configured for a given Database Instance.
    """


@dataclass
class ClusterVersion:
    """
    Cluster version.
    """

    version: str
    """
    Redis™ engine version.
    """

    end_of_life_at: Optional[datetime]
    """
    Date of End of Life.
    """

    available_settings: List[AvailableClusterSetting]
    """
    Cluster settings available to be updated.
    """

    logo_url: str
    """
    Redis™ logo url.
    """

    zone: Zone
    """
    Zone of the Redis™ Database Instance.
    """


@dataclass
class Endpoint:
    """
    Endpoint.
    """

    port: int
    """
    TCP port of the endpoint.
    """

    private_network: Optional[PrivateNetwork]
    """
    Private Network details.
    
    One-of ('details'): at most one of 'private_network', 'public_network' could be set.
    """

    public_network: Optional[PublicNetwork]
    """
    Public network details.
    
    One-of ('details'): at most one of 'private_network', 'public_network' could be set.
    """

    ips: List[str]
    """
    List of IPv4 addresses of the endpoint.
    """

    id: str
    """
    UUID of the endpoint.
    """


@dataclass
class EndpointSpec:
    """
    Endpoint spec.
    """

    private_network: Optional[EndpointSpecPrivateNetworkSpec]
    """
    Private Network specification details.
    
    One-of ('endpoint_type'): at most one of 'private_network', 'public_network' could be set.
    """

    public_network: Optional[EndpointSpecPublicNetworkSpec]
    """
    Public network specification details.
    
    One-of ('endpoint_type'): at most one of 'private_network', 'public_network' could be set.
    """


@dataclass
class EndpointSpecPrivateNetworkSpec:
    """
    Endpoint spec. private network spec.
    """

    id: str
    """
    UUID of the Private Network to connect to the Database Instance.
    """

    service_ips: List[str]
    """
    Endpoint IPv4 address with a CIDR notation. You must provide at least one IPv4 per node.
    """

    ipam_config: Optional[EndpointSpecPrivateNetworkSpecIpamConfig]
    """
    Automated configuration of your Private Network endpoint with Scaleway IPAM service.
    """


@dataclass
class EndpointSpecPrivateNetworkSpecIpamConfig:
    pass


@dataclass
class EndpointSpecPublicNetworkSpec:
    """
    Endpoint spec. public network spec.
    """


@dataclass
class ListClusterVersionsResponse:
    """
    List cluster versions response.
    """

    versions: List[ClusterVersion]
    """
    List of available Redis™ engine versions.
    """

    total_count: int
    """
    Total count of available Redis™ engine versions.
    """


@dataclass
class ListClustersResponse:
    """
    List clusters response.
    """

    clusters: List[Cluster]
    """
    List all Database Instances.
    """

    total_count: int
    """
    Total count of Database Instances.
    """


@dataclass
class ListNodeTypesResponse:
    """
    List node types response.
    """

    node_types: List[NodeType]
    """
    Types of node.
    """

    total_count: int
    """
    Total count of node types available.
    """


@dataclass
class NodeType:
    """
    Node type.
    """

    name: str
    """
    Node type name.
    """

    stock_status: NodeTypeStock
    """
    Current stock status of the node type.
    """

    description: str
    """
    Current specifications of the offer.
    """

    vcpus: int
    """
    Number of virtual CPUs.
    """

    memory: int
    """
    Quantity of RAM.
    """

    disabled: bool
    """
    Defines whether node type is currently disabled or not.
    """

    beta: bool
    """
    Defines whether node type is currently in beta.
    """

    zone: Zone
    """
    Zone of the node type.
    """


@dataclass
class PrivateNetwork:
    """
    Private network.
    """

    id: str
    """
    UUID of the Private Network.
    """

    service_ips: List[str]
    """
    List of IPv4 CIDR notation addresses of the endpoint.
    """

    zone: Zone
    """
    Zone of the Private Network.
    """


@dataclass
class PublicNetwork:
    pass


@dataclass
class SetAclRulesResponse:
    """
    Set acl rules response.
    """

    acl_rules: List[ACLRule]
    """
    ACL Rules enabled for the Database Instance.
    """


@dataclass
class SetEndpointsResponse:
    """
    Set endpoints response.
    """

    endpoints: List[Endpoint]
    """
    Endpoints defined on the Database Instance.
    """


@dataclass
class CreateClusterRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    project_id: Optional[str]
    """
    Project ID in which to create the Database Instance.
    """

    name: Optional[str]
    """
    Name of the Database Instance.
    """

    version: str
    """
    Redis™ engine version of the Database Instance.
    """

    tags: Optional[List[str]]
    """
    Tags to apply to the Database Instance.
    """

    node_type: str
    """
    Type of node to use for the Database Instance.
    """

    user_name: str
    """
    Name of the user created upon Database Instance creation.
    """

    password: str
    """
    Password of the user.
    """

    cluster_size: Optional[int]
    """
    Number of nodes in the Redis™ cluster.
    """

    acl_rules: Optional[List[ACLRuleSpec]]
    """
    List of ACLRuleSpec used to secure your publicly exposed cluster.
    """

    endpoints: Optional[List[EndpointSpec]]
    """
    Zero or multiple EndpointSpec used to expose your cluster publicly and inside Private Networks.
    Zero or multiple EndpointSpec used to expose your cluster publicly and inside private networks. If no EndpoindSpec is given the cluster will be publicly exposed by default.
    """

    tls_enabled: bool
    """
    Defines whether or not TLS is enabled.
    """

    cluster_settings: Optional[List[ClusterSetting]]
    """
    List of advanced settings to be set upon Database Instance initialization.
    """


@dataclass
class UpdateClusterRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance to update.
    """

    name: Optional[str]
    """
    Name of the Database Instance.
    """

    tags: Optional[List[str]]
    """
    Database Instance tags.
    """

    user_name: Optional[str]
    """
    Name of the Database Instance user.
    """

    password: Optional[str]
    """
    Password of the Database Instance user.
    """


@dataclass
class GetClusterRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the cluster.
    """


@dataclass
class ListClustersRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    tags: Optional[List[str]]
    """
    Filter by Database Instance tags.
    """

    name: Optional[str]
    """
    Filter by Database Instance names.
    """

    order_by: Optional[ListClustersRequestOrderBy]
    """
    Criteria to use when ordering the list.
    """

    project_id: Optional[str]
    """
    Filter by Project ID.
    """

    organization_id: Optional[str]
    """
    Filter by Organization ID.
    """

    version: Optional[str]
    """
    Filter by Redis™ engine version.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class MigrateClusterRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance to update.
    """

    version: Optional[str]
    """
    Redis™ engine version of the Database Instance.
    
    One-of ('action'): at most one of 'version', 'node_type', 'cluster_size' could be set.
    """

    node_type: Optional[str]
    """
    Type of node to use for the Database Instance.
    
    One-of ('action'): at most one of 'version', 'node_type', 'cluster_size' could be set.
    """

    cluster_size: Optional[int]
    """
    Number of nodes for the Database Instance.
    
    One-of ('action'): at most one of 'version', 'node_type', 'cluster_size' could be set.
    """


@dataclass
class DeleteClusterRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance to delete.
    """


@dataclass
class GetClusterMetricsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the cluster.
    """

    start_at: Optional[datetime]
    """
    Start date.
    """

    end_at: Optional[datetime]
    """
    End date.
    """

    metric_name: Optional[str]
    """
    Name of the metric to gather.
    """


@dataclass
class ListNodeTypesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    include_disabled_types: bool
    """
    Defines whether or not to include disabled types.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class ListClusterVersionsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    include_disabled: bool
    """
    Defines whether or not to include disabled Redis™ engine versions.
    """

    include_beta: bool
    """
    Defines whether or not to include beta Redis™ engine versions.
    """

    include_deprecated: bool
    """
    Defines whether or not to include deprecated Redis™ engine versions.
    """

    version: Optional[str]
    """
    List Redis™ engine versions that match a given name pattern.
    """

    page: Optional[int]

    page_size: Optional[int]


@dataclass
class GetClusterCertificateRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the cluster.
    """


@dataclass
class RenewClusterCertificateRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the cluster.
    """


@dataclass
class AddClusterSettingsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance you want to add settings to.
    """

    settings: List[ClusterSetting]
    """
    Settings to add to the cluster.
    """


@dataclass
class DeleteClusterSettingRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance where the settings must be set.
    """

    setting_name: str
    """
    Setting name to delete.
    """


@dataclass
class SetClusterSettingsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance where the settings must be set.
    """

    settings: List[ClusterSetting]
    """
    Settings to define for the Database Instance.
    """


@dataclass
class SetAclRulesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance where the ACL rules have to be set.
    """

    acl_rules: List[ACLRuleSpec]
    """
    ACLs rules to define for the cluster.
    """


@dataclass
class AddAclRulesRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance you want to add ACL rules to.
    """

    acl_rules: List[ACLRuleSpec]
    """
    ACLs rules to add to the cluster.
    """


@dataclass
class DeleteAclRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    acl_id: str
    """
    UUID of the ACL rule you want to delete.
    """


@dataclass
class GetAclRuleRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    acl_id: str
    """
    UUID of the ACL rule you want to get.
    """


@dataclass
class SetEndpointsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance where the endpoints have to be set.
    """

    endpoints: List[EndpointSpec]
    """
    Endpoints to define for the Database Instance.
    """


@dataclass
class AddEndpointsRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    cluster_id: str
    """
    UUID of the Database Instance you want to add endpoints to.
    """

    endpoints: List[EndpointSpec]
    """
    Endpoints to add to the Database Instance.
    """


@dataclass
class DeleteEndpointRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    endpoint_id: str
    """
    UUID of the endpoint you want to delete.
    """


@dataclass
class GetEndpointRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    endpoint_id: str
    """
    UUID of the endpoint you want to get.
    """


@dataclass
class UpdateEndpointRequest:
    zone: Optional[Zone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    endpoint_id: str

    private_network: Optional[EndpointSpecPrivateNetworkSpec]
    """
    One-of ('endpoint_type'): at most one of 'private_network', 'public_network' could be set.
    """

    public_network: Optional[EndpointSpecPublicNetworkSpec]
    """
    One-of ('endpoint_type'): at most one of 'private_network', 'public_network' could be set.
    """
